package termites;

import termites.agent.Termite;
import termites.agent.Wood;
import fr.lifl.jedi.controllersCore.AbstractControlGUIController;
import fr.lifl.jedi.controllersCore.simulationRun.SimulationCore;
import fr.lifl.jedi.model.Environment;
import fr.lifl.jedi.util.SimulationProperties;

/**
 * This class defines how the simulation is initialized, and when the simulation
 * is considered as finished.
 * 
 * @author
 *	<table>
 *		<tr>
 *			<td>Yoann Kubera</td>
 *		</tr>
 *		<tr>
 *			<td><a href="http://www.lifl.fr/SMAC/">SMAC Team (Systèmes Multi-Agents et Comportement)</a></td>
 *		</tr>
 *		<tr>
 *			<td><a href="http://www.lifl.fr/">LIFL (Laboratoire d'Informatique Fondamentale de Lille)</a></td>
 *		</tr>
 *		<tr>
 *			<td><a href="http://www.univ-lille1.fr/">University of Lille, France</a></td>
 *		</tr>
 *  </table>
 * @version
 * 	<table>
 *		<tr>
 *			<td>JEDI V 2.3</td>
 *		</tr>
 *		<tr>
 *			<td>Created the 4th Nov. 2009</td>
 *		</tr>
 *	</table>
 */
public class TermitesSimulationCore extends SimulationCore {
	/**
	 * Default constructor of this class.
	 */
	public TermitesSimulationCore(AbstractControlGUIController<?> guiControl, Environment e) {
		super(guiControl, e);
	}

	/**
	 * This method defines the ending criterion of the simulation.
	 * Returning <code>false</code> means that the simulation never ends.
	 */
	public boolean hasToStop() {
		return false;
	}

	/**
	 * An attribute storing the initial number of termites in the simulation.
	 */
	private final int termitesNumber = 300;
	/**
	 * An attribute storing the initial number of wood chips in the simulation.
	 */
	private final int woodNumber = 3000;
	
	/**
	 * This method initializes the simulation.
	 * It instantiates all agents, and puts them into the environment.
	 */
	public void initialize() {
		// Wood chip agents creation.
		for(int i = 0; i < woodNumber; i++){
			Wood w = new Wood();			
			/* 
			 * Generate random coordinates to put the agent in the environment.
			 * 
			 * Using the SimulationProperties.random() instead of Math.random() as random
			 * number generator makes possible to reproduce exactly a simulation if required,
			 * by using the same random seed.
			 */
			double x = SimulationProperties.random() * ( environment.getWidth() - 1);
			double y = SimulationProperties.random() * ( environment.getHeight() - 1);
			/*
			 * Since wood chips are 1 x 1 squares.
			 * If we want to put such an agent in the area of 
			 * the environment bounded by the (0;0) and (1;1) points, this means that the center
			 * of the wood chip has to be located at (0.5;0.5).
			 * 
			 * Thus, we add 0.5 to the floor of each coordinates of the agent.
			 */
			x = Math.floor(x) + 0.5;
			y = Math.floor(y) + 0.5;
			if(environment.canBePutAt(w, x, y)){
				// If the agent can be put at this position, the agent
				// is put.
				environment.putAgent(w, x, y);
			} else {
				// If we correctly computed "x" and "y", this case will not occur.
				throw new RuntimeException("An unexpected error occured");
			}
		}
		// Termites agents creation.
		for(int i = 0; i < termitesNumber; i++){
			Termite t = new Termite();
			// Puts the agent in the environment
			double x = SimulationProperties.random() * environment.getWidth();
			double y = SimulationProperties.random() * environment.getHeight();
			/*
			 * Since Termites are point agents, no values have to be added to
			 * x and y.
			 */
			if(environment.canBePutAt(t, x, y)){
				// If the agent can be put at this position, the agent
				// is put.
				environment.putAgent(t, x, y);
			} else {
				// If we correctly computed "x" and "y", this case will not occur.
				throw new RuntimeException("An unexpected error occured");
			}
		}
	}
}
