package termites.agent;

import termites.interaction.PickUp;
import termites.interaction.Put;
import termites.interaction.Wander;
import termites.interaction.functions.PickUpSourceFunction;
import termites.interaction.functions.PutSourceFunction;
import fr.lifl.jedi.model.Agent;
import fr.lifl.jedi.model.halo.HaloBuilder;
import fr.lifl.jedi.model.interactionDeclaration.InteractionMatrixLine;
import fr.lifl.jedi.util.SimulationProperties;

/**
 * The java class that represents the termite agent family in the termites nest simulation.
 * 
 * @author
 *	<table>
 *		<tr>
 *			<td>Yoann Kubera</td>
 *		</tr>
 *		<tr>
 *			<td><a href="http://www.lifl.fr/SMAC/">SMAC Team (Systèmes Multi-Agents et Comportement)</a></td>
 *		</tr>
 *		<tr>
 *			<td><a href="http://www.lifl.fr/">LIFL (Laboratoire d'Informatique Fondamentale de Lille)</a></td>
 *		</tr>
 *		<tr>
 *			<td><a href="http://www.univ-lille1.fr/">University of Lille, France</a></td>
 *		</tr>
 *  </table>
 * @version 
 * 	<table>
 *		<tr>
 *			<td>JEDI V 2.3</td>
 *		</tr>
 *		<tr>
 *			<td>Created the 5th Nov. 2009</td>
 *		</tr>
 *	</table>
 */
public class Termite extends Agent 
				implements PickUpSourceFunction, // Required if this agent family can be the source of the PickUp interaction
				PutSourceFunction  // Required if this agent family can be the source of the Put interaction
{
	/**
	 * All Termites have the same interaction matrix line.
	 * Thus, to avoid multiple creations, this line is declared as a static attribute of this class.
	 */
	public static final InteractionMatrixLine TERMITE_MATRIX_LINE = createTermiteMatrixLine();
	
	/**
	 * This method creates the interaction matrix line of all termites of the simulation.
	 */
	private static InteractionMatrixLine createTermiteMatrixLine(){
		// Creation of the matrix line of agents from the Termites family.
		InteractionMatrixLine termitesLine = new InteractionMatrixLine();
		// Tells that the default interaction performed by termites is the "Wander" interaction.
		Wander wander = new Wander();
		termitesLine.add(wander, 
				0); // The priority. The higher the value is, the more likely the interaction will
					// be performed.
		PickUp pickUp = new PickUp();
		// Tell that termites are able to pick up wood agents.
		termitesLine.add(pickUp, 
				Wood.class, 
				0, // Limit distance = 0. It means that Wood chips 
				   // are picked up only if the distance between them 
				   // and the termite is 0.
				1); // The priority.
		// Tell that termites are able to put down wood agents.
		Put put = new Put();
		termitesLine.add(put, 
				Wood.class, 
				0.5, // Wood chips are put down if they are close 
					 // enough from another wood chip.
					 // "Close enough" is interpreted as a 0.5 distance.
				2); // The priority.
		return termitesLine;
	}
	
	/**
	 * Constructor of the Termite agent family.
	 * This constructor defines in which surface of the environment Termites do 
	 * perceive, which matrix line they have, and how they are represented in the 
	 * environment.
	 */
	public Termite() {
		super(
			// Termites perceive in a disc which center lies at the position of the agent,
			// and which radius is equal to 1.
			// This attribute defines such a perception.
			HaloBuilder.INSTANCE.createDiscShapedArea(1), 
			// The matrix line for this agent family.
			TERMITE_MATRIX_LINE);
		// Tells that a termite looks forward in a random direction when
		// created.
		setDirection(SimulationProperties.random() * 360 - 180);
	}

	/**
	 * This method updates the state of the agent at the beginning 
	 * of every time step.
	 * In this model, this method is not used.
	 */
	public void update() {}

	/*
	 * Elements related to the primitives that the agent family has to implement in order
	 * to participate in a simulation:
	 */
		
	// The wood chip the termite is currently carrying.
	// Null means no wood chip is carried.
	private Agent carriedAgent = null;

	/**
	 * @see termites.interaction.functions.PickUpSourceFunction#storeTheElement(Agent)
	 */
	public void storeTheElement(Agent a) {
		carriedAgent = a;
	}

	/**
	 * @see termites.interaction.functions.PickUpSourceFunction#ownsAnElement()
	 */
	public boolean ownsAnElement() {
		return carriedAgent != null;
	}

	/**
	 * @see termites.interaction.functions.PutSourceFunction#retrieveElement()
	 */
	public Agent retrieveElement() {
		Agent a = carriedAgent;
		carriedAgent = null;
		return a;
	}
}
