package termites.interaction;

import termites.interaction.functions.PickUpSourceFunction;
import fr.lifl.jedi.model.Agent;
import fr.lifl.jedi.model.Environment;
import fr.lifl.jedi.model.interactionDeclaration.SingleTargetInteraction;

/**
 * This interaction involves a source agent, and a target agent.
 * <p>
 * 	It puts the target agent in the inventory of the source agent, and removes
 * 	it from the environment.
 * 	Then, the source moves away from its current position by making a U-turn,
 * 	and moving forward in 1 units.
 * </p>
 * <p>
 * 	It can be performed only if the source agent is not already carrying something.
 * </p>
 * <p>
 * 	Primitives that the source agent has to implement:
 * 	<ul>
 * 		<li>
 * 			<code>ownsAnElement()</code> : it checks if the source agent 
 * 			is carrying a resource;
 * 		</li>
 * 		<li>
 * 			<code>storeTheElement(Agent a)</code> : it adds the target agent (provided 
 * 			as a parameter) to the inventory of the source agent.
 * 		</li>
 * 	</ul>
 * 	The source function of the interaction requires the source agent to implement two primitives.
 * 	This function is made concrete in JEDI by creating an interface called <i>PickUpSourceFunction</i>.
 * 	An agent that can be the source of this interaction has to implement this interface.
 * </p>
 * 
 * @author
 *	<table>
 *		<tr>
 *			<td>Yoann Kubera</td>
 *		</tr>
 *		<tr>
 *			<td><a href="http://www.lifl.fr/SMAC/">SMAC Team (Systèmes Multi-Agents et Comportement)</a></td>
 *		</tr>
 *		<tr>
 *			<td><a href="http://www.lifl.fr/">LIFL (Laboratoire d'Informatique Fondamentale de Lille)</a></td>
 *		</tr>
 *		<tr>
 *			<td><a href="http://www.univ-lille1.fr/">University of Lille, France</a></td>
 *		</tr>
 *  </table>
 * @version 
 * 	<table>
 *		<tr>
 *			<td>JEDI V 2.3</td>
 *		</tr>
 *		<tr>
 *			<td>Created the 4th Nov. 2009</td>
 *		</tr>
 *	</table>
 */
public class PickUp extends SingleTargetInteraction {
	/**
	 * Returns <code>true</code> only if the agent does not carry already something.
	 */
	public boolean preconditions(Environment environment, Agent source, Agent target) {
		PickUpSourceFunction csource = (PickUpSourceFunction) source;
		return ! csource.ownsAnElement();
	}

	/**
	 * Not used in this model.
	 */
	public boolean trigger(Environment environment, Agent source, Agent target) {
		return true;
	}

	/**
	 * This method performs the actions related to the PickUp interaction.
	 */
	public void perform(Environment environment, Agent source, Agent target) {
		PickUpSourceFunction csource = (PickUpSourceFunction) source;
		// Adds the target to the inventory of the source.
		csource.storeTheElement(target);
		// Removes the target from the environment.
		environment.removeAgent(target);
		// Moves the source away from its current location.
		environment.turnLeft(source, 180);
		if(environment.canMoveForward(source, 1)){
			environment.moveForward(source, 1);
		}
	}
}
