package termites.interaction;

import termites.interaction.functions.PutSourceFunction;
import fr.lifl.jedi.model.Agent;
import fr.lifl.jedi.model.Environment;
import fr.lifl.jedi.model.interactionDeclaration.SingleTargetInteraction;

/**
 * This interaction involves a source agent, and a target agent.
 * <p>
 * 	It gets the resource carried by the source agent, and puts it 
 * 	in the environment.
 * 	The position where the resource is put is computed by:
 * 	<ul>
 * 		<li>
 * 			viewing the environment as a grid, which cells are 1 x 1 units long.
 * 		</li>
 * 		<li>
 * 			putting the resource at the center of the cell the source agent is in.
 * 		</li>
 * 	</ul>
 * </p>
 * <p>
 * 	Then, the source moves away from its current position by making a U-turn,
 * 	and moving forward in 1.5 units.
 * </p>
 * <p>
 * 	This interaction can be performed only if the source agent is carrying a resource.
 * </p>
 * <p>
 * 	Primitives that the source agent has to implement:
 * 	<ul>
 * 		<li>
 * 			<code>ownsAnElement()</code> : it checks if the source agent 
 * 			is carrying a resource;
 * 		</li>
 * 		<li>
 * 			<code>Agent unstoreTheElement()</code> : removes the resource the source
 * 			agent is carrying from its inventory, and returns it.
 * 		</li>
 * 	</ul>
 * 	The source function of the interaction requires the source agent to implement two primitives.
 * 	This function is made concrete in JEDI by creating an interface called <i>PutSourceFunction</i>.
 * 	An agent that can be the source of this interaction has to implement this interface.
 * </p>
 * 
 * @author
 *	<table>
 *		<tr>
 *			<td>Yoann Kubera</td>
 *		</tr>
 *		<tr>
 *			<td><a href="http://www.lifl.fr/SMAC/">SMAC Team (Systèmes Multi-Agents et Comportement)</a></td>
 *		</tr>
 *		<tr>
 *			<td><a href="http://www.lifl.fr/">LIFL (Laboratoire d'Informatique Fondamentale de Lille)</a></td>
 *		</tr>
 *		<tr>
 *			<td><a href="http://www.univ-lille1.fr/">University of Lille, France</a></td>
 *		</tr>
 *  </table>
 * @version 
 * 	<table>
 *		<tr>
 *			<td>JEDI V 2.3</td>
 *		</tr>
 *		<tr>
 *			<td>Created the 4th Nov. 2009</td>
 *		</tr>
 *	</table>
 */
public class Put extends SingleTargetInteraction {
	/**
	 * This interaction can be performed only if the source agent carries something.
	 */
	public boolean preconditions(Environment environment, Agent source, Agent target) {
		PutSourceFunction csource = (PutSourceFunction) source;
		return csource.ownsAnElement();
	}

	/**
	 * Not used in this model.
	 */
	public boolean trigger(Environment environment, Agent source, Agent target) {
		return true;
	}

	/**
	 * This method performs the actions related to the PickUp interaction.
	 */
	public void perform(Environment environment, Agent source, Agent target) {
		PutSourceFunction csource = (PutSourceFunction) source;
		// Gets the element carried by the source agent.
		Agent a = csource.retrieveElement();
		// Computes the position where to put the element.
		double x = Math.floor(source.getXPosition()) + 0.5;
		double y = Math.floor(source.getYPosition()) + 0.5;
		if(environment.canBePutAt(a, x, y)){
			environment.putAgent(a, x, y);
		}
		// Gets away from the position where the wood chip was put.
		environment.turnLeft(source, 180);
		if(environment.canMoveForward(source, 1.5)){
			environment.moveForward(source, 1.5);
		}
	}
}
